#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
from ionas import request
from iobase.exception import (NotFoundError,)
from libfal import falcmd
import pal_info
import fal_config
from fal_info import (ReqTypes, Results)
import product_conf
from iobase import fileut
from iobase.xmldata import (XmlNode, XmlConfNode)

from PIL import Image
from PIL.ExifTags import TAGS


def _dict_merge(d1, d2):
    d = {}
    for k in d1.keys() + d2.keys():  # FIXME: 重複
        if k in d1:
            v1 = d1[k]
            if k in d2:
                v2 = d2[k]
                if isinstance(v1, dict) and isinstance(v2, dict):
                    d[k] = _dict_merge(v1, v2)
                else:
                    d[k] = v2
            else:
                d[k] = v1
        else:
            v2 = d2[k]
            d[k] = v2
    return d


def _make_thumbnail(input_file, output_file):
    img = Image.open(input_file, "r")
    size = (100, 100)
    img.thumbnail(size)
    img.save(output_file, "JPEG")


def _get_exif(input_file):
    exif = {}
    img = Image.open(input_file)
    try:
        _exif = img._getexif()
    except AttributeError:
        return exif
    for k, v in _exif.iteritems():
        exif[TAGS.get(k, k)] = v
    return exif


class FALContents(falcmd.FalCmd):

    def __thumbnail_handler(self, entry):
        """

        <entry type="thumbnail">
            <contents>
                <file>
                    <path>/path/to/mmfile.mp4</path>
                    <mminfo>
                        <thumbnail>
                            <path>/path/to/thumbnailfile.png</path>
                        </thumbnail>
                    </mminfo>
                </file>
            </contents>
        </entry>

        <entry type="thumbnail" result="0">
            <contents>
                <file>
                    <path>/path/to/mmfile.mp4</path>
                    <mminfo>
                        <thumbnail>
                            <path>/path/to/thumbnailfile.png</path>
                        </thumbnail>
                    </mminfo>
                </file>
            </contents>
        </entry>

        Results.OK
        Results.???
        """
        entry.set_result(Results.INTERNAL_ERROR)  # FIXME

        contents_node = entry.get()

        input_file = contents_node.get_value("file/path")
        output_file = contents_node.get_value("file/mminfo/thumbnail/path")

        try:
            _make_thumbnail(input_file, output_file)
            entry.set_result(Results.OK)
        except:
            entry.set_result(Results.INTERNAL_ERROR)  # FIXME

        return entry

    def __query_handler(self, entry):
        """

        <entry type="query">
            <contents>
                <file>
                    <path>/path/to/file</path>
                </file>
            </contents>
        </entry>

        <entry type="query" result="0">
            <contents>
                <file>
                    <path>/path/to/file</path>
                    <mminfo>
                        <exif>
                            <imagedescription>...</imagedescription>
                            <make>...</make>
                            <orientation>...</orientation>
                            <xresolution>...</xresolution>
                            <yresolution>...</yresolution>
                            <resolutionunit>...</resolutionunit>
                            <software>...</software>
                            <datetime>...</datetime>
                            <copyright>...</copyright>
                            <datetimeoriginal>...</datetimeoriginal>
                            <compressedbitsperpixel>...</compressedbitsperpixel>
                            <shutterspeedvalue>...</shutterspeedvalue>
                            <apperturevalue>...</apperturevalue>
                            <brightnessvalue>...</brightnessvalue>
                            <exposurebiasvalue>...</exposurebiasvalue>
                            <maxaperturevalue>...</maxaperturevalue>
                            <subjectdistance>...</subjectdistance>
                            <meteringmode>...</meteringmode>
                            <lightsource>...</lightsource>
                            <flash>...</flash>
                            <exifimagewidth>...</exifimagewidth>
                            <exifimageheight>...</exifimageheight>
                        </exif>
                    </mminfo>
                </file>
            </contents>
        </entry>

        Results.OK
        Results.???
        """
        entry.set_result(Results.INTERNAL_ERROR)  # FIXME

        contents_node = entry.get()

        input_file = contents_node.get_value("file/path")

        contents_node.append_child("file")
        file_node = contents_node.get_node("file")
        file_node.append_child("mminfo")
        mminfo_node = file_node.get_node("mminfo")
        mminfo_node.append_child("exif")
        exif_node = mminfo_node.get_node("exif")
        for k, v in _get_exif(input_file).iteritems():
            if k in ("ImageDescription",
                     "Make",
                     "Orientation",
                     "XResolution",
                     "YResolution",
                     "ResolutionUnit",
                     "Software",
                     "DateTime",
                     "Copyright",
                     "DateTimeOriginal",
                     "CompressedBitsPerPixel",
                     "ShutterSpeedValue",
                     "ApertureValue",
                     "BrightnessValue",
                     "ExposureBiasValue",
                     "MaxApertureValue",
                     "SubjectDistance",
                     "MeteringMode",
                     "LightSource",
                     "Flash",
                     "ExifImageWidth",
                     "ExifImageHeight"):
                exif_node.append_child(k.lower(), str(v))
        entry.set_result(Results.OK)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler("thumbnail", self.__thumbnail_handler)
        self.set_handler("query", self.__query_handler)

#
# Main
#

if __name__ == "__main__":
    FALContents(option=falcmd.FalCmdOption()).handle_request()
