#!/bin/sh
FIFO=/var/log/rl3.log.fifo
LOGD=/opt/rl3/bin/logd

RL3_HOME=/opt/rl3/bin
TOU_SERVER=./tou_server
DEVICE_PROCESS=./device_process

WATCHDOGD_NAME=`basename $0`

kill_pid() {
	local PID="$*"
	echo "${WATCHDOGD_NAME} try to kill ${PID}" > "${FIFO}"
	kill -9 ${PID}
}

term_pid() {
	local PID="$*"
	echo "${WATCHDOGD_NAME} try to terminate ${PID}" > "${FIFO}"
	kill -15 ${PID}
}

get_alive_process_count() {
	local PROCESS="$1"
	COUNT=`pgrep -f ${PROCESS} | wc -l`
	[ "${COUNT}" = '' ] && COUNT=0
	echo -n "${COUNT}"
}

start_process() {
	local PROCESS="$1"
	local PROCESS_NAME=`basename ${PROCESS}`
	${PROCESS} & > /dev/null 2>&1
	echo "${WATCHDOGD_NAME} start ${PROCESS_NAME}." > "${FIFO}"
}

term_process() {
	local PROCESS="$1"
	local PROCESS_NAME=`basename ${PROCESS}`
	local PID=`pgrep -f "${PROCESS}"`
	term_pid ${PID}
}

kill_process() {
	local PROCESS="$1"
	local PROCESS_NAME=`basename ${PROCESS}`
	local PID=`pgrep -f "${PROCESS}"`
	kill_pid ${PID}
}

stop_process() {
	local PROCESS="$1"
	local PROCESS_NAME=`basename "${PROCESS}"`
	echo "${WATCHDOGD_NAME} stopping ${PROCESS_NAME}." > "${FIFO}"
	term_process "${PROCESS}"
	local CHECK_LIMIT=30
	local CHECK_COUNT=0
	while true; do
		ALIVE_PROCESS_COUNT=`get_alive_process_count "${PROCESS}"`
		if [ ${ALIVE_PROCESS_COUNT} -eq 0 ]; then
			break
		fi
		echo "${WATCHDOGD_NAME} waiting for terminate:alive=${ALIVE_PROCESS_COUNT}" > "${FIFO}"
		sleep 1
		CHECK_COUNT=$((CHECK_COUNT+1))
		if [ ${CHECK_COUNT} -ge ${CHECK_LIMIT} ]; then
			echo "${WATCHDOGD_NAME} GIVE UP." > "${FIFO}"
			kill_process "${PROCESS}"
			break
		fi
	done
	echo "${WATCHDOGD_NAME} stopped ${PROCESS_NAME}." > "${FIFO}"
}

CURRENT_HOME=`pwd`
CURRENT_LD_LIBRARY_PATH="${LD_LIBRARY_PATH}"
export LD_LIBRARY_PATH="${RL3_HOME}"
cd "${RL3_HOME}"
while pgrep -f ${LOGD}; do
	sleep 1
	ALIVE_TOU_SERVER_COUNT=`get_alive_process_count ${TOU_SERVER}`
	if [ ${ALIVE_TOU_SERVER_COUNT} -eq 0 ]; then
		echo "${WATCHDOGD_NAME} DETECTED ISSUE." > "${FIFO}"
		#stop_process "${DEVICE_PROCESS}"
		kill_process "${DEVICE_PROCESS}"
		start_process "${TOU_SERVER}"
		start_process "${DEVICE_PROCESS}"
	fi
done
export LD_LIBRARY_PATH="${CURRENT_LD_LIBRARY_PATH}"
cd "${CURRENT_HOME}"
