#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import sys
import shutil
import signal
import rapscomm
import errno
import pwd

# initalize
c_comm   = rapscomm.rapscomm()
n_result = c_comm.RESULT.RESULT_NO_ERROR

# set permission
def setpms(dest, uid, gid, mode):
    if pwd.getpwuid(os.geteuid()).pw_name != "root":
        return
    try:
        # chmod
        os.chmod(dest, mode)
    except:
        pass
    try:
        # chown
        os.chown(dest, uid, gid)
    except:
        pass
    return

# make file
def mkfile(dest, uid, gid):
    try:
        os.remove(dest)
    except:
        pass
    try:
        # create
        fp = open(dest , "wb") 
        fp.close()
    except:
        raise

    # set permission
    setpms(dest, uid, gid, c_comm.CONFIG.PERMISSION)

    return

# makedir
def mkdirs(dest, uid, gid):
    head, tail = os.path.split(dest)
    if not tail:
        head, tail = os.path.split(head)
    if head and tail and not os.path.exists(head):
        try:
            mkdirs(head)
        except OSError, e:
            if e.errno != errno.EEXIST:
                raise
        if tail == os.curdir:
            return
    try:
        os.mkdir(dest)
    except OSError, e:
        if e.errno != errno.EEXIST:
            raise

    # set permission
    setpms(dest, uid, gid, c_comm.CONFIG.DIRPERMISS)

# dircopy
def dircopy(src, dst, uid, gid):
    if os.path.exists(dst) == False:
        try:
            mkdirs(dst, uid, gid)
        except:
            raise

    names = os.listdir(src)
    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if os.path.islink(srcname):
                try:
                    linkto = os.readlink(srcname)
                    os.symlink(linkto, dstname)
                except:
                    pass
            elif os.path.isdir(srcname):
                dircopy(srcname, dstname, uid, gid)
            else:
                mkfile(dstname, uid, gid)
                shutil.copy2(srcname, dstname)
                # set permission
                setpms(dstname, uid, gid, c_comm.CONFIG.PERMISSION)
        except Exception:
            raise

    return

# output exit code
def exitout():
    sys.stdout.write(str(n_result))
    sys.stdout.write("\n")
    sys.exit(0)

# set signal function
def sigfunc(num, frame):
    n_result = c_comm.RESULT.RESULT_ERROR
    exitout()

signal.signal( signal.SIGTERM, sigfunc)

# parameter check
if len(sys.argv) < 5:
    n_result = c_comm.RESULT.RESULT_INVALID_OPTION
    exitout()

# get parameter
i_uid    = int(sys.argv[1])
i_gid    = int(sys.argv[2])
s_src    = sys.argv[3]
s_dest   = sys.argv[4]

# read config
if c_comm.readConfig() == False:
    n_result = c_comm.RESULT.RESULT_ERROR
    exitout()

# set filename
real_dst = s_dest
if os.path.isdir(s_dest):
    real_dst = os.path.join(s_dest, os.path.basename(s_src))

# copy
if os.path.isfile(s_src) == True:
    try:
        # copy file and directory
        mkfile(real_dst, i_uid, i_gid)
        shutil.copy2(s_src, real_dst)
        # set permission
        setpms(real_dst, i_uid, i_gid, c_comm.CONFIG.PERMISSION)

    except (IOError, OSError), e:
        if  e.errno == errno.ENOENT:
            n_result = c_comm.RESULT.RESULT_NOT_FOUND
        elif e.errno == errno.EACCES:
            n_result = c_comm.RESULT.RESULT_ERR_PERMISSION
        else:
            n_result = c_comm.RESULT.RESULT_ERR_WRITE

    except Exception ,e:
        n_result = c_comm.RESULT.RESULT_ERR_WRITE

else:
    if s_src == real_dst:
        n_result = c_comm.RESULT.RESULT_ERR_INVALID_FILE
    if real_dst.startswith(s_src + '/'):
        n_result = c_comm.RESULT.RESULT_ERR_INVALID_FILE
    else:
        try:
            # copy file and directory
            dircopy(s_src, real_dst, i_uid, i_gid)

        except (IOError, OSError), e:
            if  e.errno == errno.ENOENT:
                n_result = c_comm.RESULT.RESULT_NOT_FOUND
            elif e.errno == errno.EACCES:
                n_result = c_comm.RESULT.RESULT_ERR_PERMISSION
            else:
                n_result = c_comm.RESULT.RESULT_ERR_WRITE

        except Exception ,e:
            n_result = c_comm.RESULT.RESULT_ERR_WRITE

exitout()
