#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import sys
import stat
import signal
import rapscomm


# initalize
c_comm = rapscomm.rapscomm()
n_result = c_comm.RESULT.RESULT_NO_ERROR


# output exit code
def exitout():
    sys.stdout.write(str(n_result))
    sys.stdout.write("\n")
    sys.exit(0)


# set signal function
def sigfunc(num, frame):
    n_result = c_comm.RESULT.RESULT_ERROR
    exitout()

signal.signal(signal.SIGTERM, sigfunc)

# parameter check
if len(sys.argv) < 2:
    n_result = c_comm.RESULT.RESULT_INVALID_OPTION
    exitout()

# get parameter
s_path = sys.argv[1]

# set mode with keepind sgid
try:
    if os.path.isfile(s_path):
        mode = os.stat(s_path).st_mode
        sgid = mode & stat.S_ISGID
        os.chmod(s_path, sgid | c_comm.CONFIG.PERMISSION)
    else:
        for curDir, dirs, files in os.walk(s_path):
            mode = os.stat(curDir).st_mode
            sgid = mode & stat.S_ISGID
            os.chmod(curDir, sgid | c_comm.CONFIG.DIRPERMISS)
            for _file in files:
                file_path = os.path.join(curDir, _file)
                mode = os.stat(file_path).st_mode
                sgid = mode & stat.S_ISGID
                os.chmod(file_path, sgid | c_comm.CONFIG.PERMISSION)
except:
    pass

exitout()
