# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.conf import settings
from project.view import FuncView, MenuView
import os
import fields
from fields import (IOPortalAccountNameTextField,
                    IOPortalPasswordField,
                    IOSerialNumberTextField)
from project.fal.remotelink3 import FALRemoteLink3
from setting.views import RL3SettingView


def _agreed():
    try:
        return bool(FALRemoteLink3().load()["data"]["terms"]["agree"])
    except:
        return False

def _agree():
    try:
        from fal_info import Results

        res = FALRemoteLink3().apply(
            {
                "terms": {
                    "agree": True,
                },
            },
            messages={
                Results.OK: ("MSG_RL3_TERMS_AGREE", {}),
                "default": ("ERR_RL3_TERMS_AGREE", {}),
            })
        success = res["success"]
        message = res["message"]
        success = success if isinstance(success, bool) else False
        message = message if isinstance(message, basestring) else u""
        return success, message
    except:
        return False, u""

def _register(name, password, serialnumber):
    try:
        from fal_info import Results

        res = FALRemoteLink3().apply(
            {
                "ioportal": {
                    "account": {
                        "name": name,
                        "password": password,
                    },
                    "product": {
                        "serialnumber": serialnumber,
                    },
                },
            },
            messages={
                Results.OK: ("MSG_RL3_REGISTER_IOPORTAL", {}),
                Results.AUTH_ERROR: ("ERR_RL3_IOPORTAL_AUTH", {}),
                Results.INVALID_DATA: ("ERR_RL3_IOPORTAL_INVALID_MAC_SN", {}),
                Results.CONNECT_ERROR: ("ERR_RL3_SERVER_CONNECT", {}),
                "default": ("ERR_RL3_REGISTER_IOPORTAL", {}),
            })
        success = res["success"]
        message = res["message"]
        success = success if isinstance(success, bool) else False
        message = message if isinstance(message, basestring) else u""
        return success, message
    except:
        return False, u""

def _registered():
    try:
        if FALRemoteLink3().load()["data"]["ioportal"]["account"]["name"]:
            return True
        return False
    except:
        return False


class RemoteLink3MenuView(MenuView):

    def view(self, request, *args, **kwargs):
        if not _agreed():
            return {self.KEY_REDIRECT: './step1'}  # FIXME: reverse()
        if not _registered():
            return {self.KEY_REDIRECT: './step2'}  # FIXME: reverse()
        return super(RemoteLink3MenuView, self).view(request, *args, **kwargs)


class RemoteLink3UserPolicyForm(forms.Form):
    from project.forms.fields.extras.agree import AgreeField
    agree = AgreeField(label=_(u'agree'))


class RemoteLink3UserPolicyView(FuncView):

    views = ['apply']

    def view(self, request):
        form = RemoteLink3UserPolicyForm(initial=self.__get_initial())

        template = os.path.join(self.TEMPLATES, 'userpolicy.html')
        dictionary = {
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        form = RemoteLink3UserPolicyForm(data=request.POST)

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            if form.is_valid():
                if form.cleaned_data["agree"]:
                    result, message = _agree()
                    proc.result(result)
                    self.set_message(message)
                else:
                    self.set_message(u"Please agree to the Remote Link 3 Terms of Use.")

        if _agreed():
            return {
                self.KEY_REDIRECT: './menu',  # FIXME: reverse()
            }

        template = os.path.join(self.TEMPLATES, 'userpolicy.html')
        dictionary = {
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __get_initial(self):
        try:
            return {
                'agree': bool(0),
            }
        except:
            return {}


class RemoteLink3IOPortalForm(forms.Form):

    account = IOPortalAccountNameTextField()
    password = IOPortalPasswordField()
    serial = IOSerialNumberTextField(label=_(u"serial number of this product"))


class RemoteLink3IOPortalView(FuncView):

    IN_INTRODUCTION = True

    views = ['apply']

    def view(self, request):
        form = RemoteLink3IOPortalForm(initial=self.__get_initial())

        template = os.path.join(self.TEMPLATES, 'ioportal.html')
        dictionary = {
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        form = RemoteLink3IOPortalForm(data=request.POST)

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            if form.is_valid():
                cleaned_data = form.cleaned_data
                result, message = _register(cleaned_data["account"], cleaned_data["password"], cleaned_data["serial"])
                proc.result(result)
                self.set_message(message)

        if self.IN_INTRODUCTION and _registered():
            return {
                self.KEY_REDIRECT: './step3',  # FIXME: reverse()
            }

        template = os.path.join(self.TEMPLATES, 'ioportal.html')
        dictionary = {
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __get_initial(self):
        try:
            data = FALRemoteLink3().load()["data"]
            return {
                "account": data["ioportal"]["account"]["name"],
                "password": data["ioportal"]["account"]["password"],
                "serial": data["ioportal"]["product"]["serialnumber"],
            }
        except:
            return {}

class RemoteLink3SettingView(RL3SettingView):

    IN_INTRODUCTION = True
